import os
from flask import Flask

def create_app(test_config=None):
    # create and configure the app

    app = Flask(__name__, instance_relative_config=True) # Creates a Flask inference
    # __name__ is a convenient way to tell Python where the current Python module is located
    # instance_relative_config tells Flask that the configuration files are relative to the instance folder


    app.config.from_mapping( # sets some default configurations
        SECRET_KEY='dev',
        # a key that keeps data safe, this SHOULD be a random number but set to 'dev' for the sake of simplicity
        DATABASE=os.path.join(app.instance_path, 'flaskr.sqlite'),
        # the database will be stored under the instance path and named 'flaskr.sqlite'
    )

    if test_config is None:
        # load the instance config, if it exists, when not testing
        app.config.from_pyfile('config.py', silent=True)
        # a 'config.py' in the instance folder can override all default configurations, which can make the SECRET_KEY actually secret
    else:
        # load the test config if passed in
        app.config.from_mapping(test_config)

    # ensure the instance folder exists
    try:
        os.makedirs(app.instance_path)
    except OSError:
        pass

    # a simple page that says hello
    @app.route('/hello')
    # route() creates a simple route for your webpage so you can see it in your browser
    # search "route", I don't want to waste comments

    def hello():
        return 'Hello, World!'
        # shows the text returned in the webpage

    from . import db
    db.init_app(app)
    # this piece of code calls db.py and inits the database, can you see how they interact?

    return app