import sqlite3
import click
from flask import current_app, g
# I got the new 4Gs on the G~
# g is a special object unique for every request, used to store data for multiple functions that might be used
# current_app is another special object that points to the current app, duh!


def get_db():
    if 'db' not in g:
        g.db = sqlite3.connect( # I think you can understand this now, Jayden
            current_app.config['DATABASE'], # it links to __init__.py! Can you understand?
            detect_types=sqlite3.PARSE_DECLTYPES
        )
        g.db.row_factory = sqlite3.Row # tells the connection to return data by rows that behave like dicts

    return g.db


def close_db(e=None): # very easy to understand
    db = g.pop('db', None)

    if db is not None:
        db.close()


def init_db():
    db = get_db()

    with current_app.open_resource('schema.sql') as f: # open_resource() opens a file RELATIVE to the flaskr folder
        # schema.sql! we've just created that!
        db.executescript(f.read().decode('utf8'))


@click.command('init-db') # 'click' is NOT a mouse click, it defines a commandline command
def init_db_command():
    """Clear the existing data and create new tables."""
    init_db()
    click.echo('Initialized the database.')

def init_app(app):
    app.teardown_appcontext(close_db) # tells flask to call the function when it has done a response and needs to clean up
    app.cli.add_command(init_db_command) # adds a command that can be called using 'flask' in the terminal
